;; X interface to the Tags facility for Emacs.
;; Copyright (C) 1992  Brad Mears

;; This file is part of taglist

;; Taglist is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; The GNU General Public License can be obtained via anonymous ftp from
;; prep.ai.mit.edu as pub/gnu/COPYING or pub/gnu/COPYING-2.

;; Taglist is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;;;
;;;  taglist.el  -  spawns the taglist sub-process
;;;  ---------------------------------------------
;;;
(defvar taglist-program "taglist"
  "*The taglist processor")

(defvar taglist-running nil
  "Is the program running?")

(defvar tl-argument ""
  "Save the called argument")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  taglist
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun taglist (&optional argument)
  "Start the taglist processor.  This starts an X subprocess that can be
used to issue commands to the current session.  By default, it will read the
contents of ./TAGLIST for the list of TAGS to use."
  (interactive)
  (if (not taglist-running)
    (progn
      (message "Starting taglist...")
      (if argument
        (setq tl-argument argument)
        (setq tl-argument "TAGLIST"))
      (setq taglist-process
        (start-process "taglist" nil taglist-program tl-argument))
      (setq taglist-running t)
      (setq taglist-current-directory default-directory)
      (set-process-filter taglist-process 'tl-process-filter)
      (set-process-sentinel taglist-process 'tl-process-sentinel)
      (process-kill-without-query taglist-process)
      (visit-tags-table "TAGS"))
    (progn
      (if (not (string-equal taglist-current-directory default-directory))
        (taglist-rescan)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  taglist-rescan
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun taglist-rescan ()
  "Force the taglist process to re-read ./TAGLIST and force emacs to revisit
the ./TAGS file."
  (interactive)
  (message "Sending rescan signal to taglist...")
  (setq taglist-current-directory default-directory)
  (process-send-string taglist-process
          (concat "1 " default-directory "TAGLIST\n"))
  (visit-tags-table "TAGS"))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  taglist-toggle-screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun taglist-toggle-screen ()
  "Sends a signal to the taglist process to change the visibilty of its
display screen.  This allows you to hide & recall the taglist screen from
within emacs.  If taglist is not running, this will start it."
  (interactive)
  (if (not taglist-running)
    (taglist)
    (progn
      (message "Sending taglist a ToggleVisibility signal...")
      (process-send-string taglist-process "2\n"))
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  tl-process-filter    - input process filter
;;;  tl-process-sentinel  - exit process filter
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun tl-process-filter (proc str)
  "Process filter for the taglist"
  (let ((first-char (string-to-char str)))
     (if (= first-char ?()     
        (eval (read str))       ; evaluate the elisp code sent from taglist
        (progn                  ; Dump the string to the taglist buffer 
           (get-buffer-create "*taglist*")
           (switch-to-buffer-other-window "*taglist*")
           (insert-string str)
           (end-of-buffer)
           (other-window 1)))))


(defun tl-process-sentinel (proc msg)
  (setq taglist-running nil)		; it's not running any more
  (cond ((eq (process-status proc) 'exit)
	 (message "taglist-proc subprocess exited"))
 	((eq (process-status proc) 'signal)
	 (message "taglist-proc subprocess killed")
	 ))) 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;  End of taglist.el
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


